/* メイン画面 */
import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


public class MainFrame extends JFrame implements ActionListener, ComponentListener {
	public static MainFrame instance;	// 唯一のインスタンス（main で作られる）

	/* メニューバー */
	private JMenuBar mnu = new JMenuBar();
	private JMenu mnuFile, mnuHelp;
	private JMenuItem mnuLoad, mnuSave, mnuExit, mnuWebpage, mnuAbout;

	/* ツールバー */
	private JToolBar tlb = new JToolBar();
	private JPanel tlbPanel = new JPanel();
	private JButton tlbShuffle = new JButton("シャッフル (S)"), tlbLast = new JButton("直近の結果 (L)"), tlbHistory = new JButton();
	private Deque<Shuffler> history = new LinkedList<Shuffler>();

	/* その他のコンポーネント */
	private JPanel west = new JPanel(), center = new JPanel();
	private JScrollPane scrollPane = new JScrollPane(center, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	private CardLayout cardLayout = new CardLayout();
	public Page activePage;
	private JFileChooser dialog;
	private javax.swing.filechooser.FileNameExtensionFilter iniFilter = new javax.swing.filechooser.FileNameExtensionFilter("設定ファイル(*.ini)", "ini");
	public SettingPage settings;

	/* コンストラクタ */
	public MainFrame(Page[] pages){
		/* 変数等の初期化 */
		instance = this;
		settings = (SettingPage)pages[0];
		addWindowListener(unloadAdapter);
		addComponentListener(this);

		/* フレームの設定・表示 */
		setTitle("Dominion サプライ選択装置 2.41");
		setSize(800, 600);
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(screen.x + screen.width / 2  - 400, screen.y + screen.height / 2 - 300);
		setLayout(new BorderLayout());

		/* Mac OS X 用の設定 */
		if (DominionSupply.isMac){
			try {
				Class cls = Class.forName("MacOSXSupporter");
				java.lang.reflect.Method mtd = cls.getMethod("setup", new Class[]{ this.getClass() });
				mtd.invoke(null, new Object[]{ this });
			} catch (Exception ex){
				ex.printStackTrace();
				DominionSupply.isMac = false;
			}
		}

		/* メニューバーの設定・表示 */
		if (!DominionSupply.isMac){
			setJMenuBar(mnu);
			mnu.add(mnuFile = new JMenu("ファイル (F)")); mnuFile.setMnemonic(KeyEvent.VK_F);
				mnuFile.add(mnuLoad = new JMenuItem("設定ファイルを開く (O)..."));
				mnuFile.add(mnuSave = new JMenuItem("設定をファイルに保存 (S)..."));
				mnuFile.addSeparator();
				mnuFile.add(mnuExit = new JMenuItem("終了 (X)"));
				mnuLoad.addActionListener(this); mnuLoad.setMnemonic(KeyEvent.VK_O);
				mnuSave.addActionListener(this); mnuSave.setMnemonic(KeyEvent.VK_S);
				mnuExit.addActionListener(this); mnuExit.setMnemonic(KeyEvent.VK_X);
			mnu.add(mnuHelp = new JMenu("ヘルプ (H)")); mnuHelp.setMnemonic(KeyEvent.VK_H);
				mnuHelp.add(mnuWebpage = new JMenuItem("ブラウザでサポートページを開く (W)"));
				mnuHelp.add(mnuAbout = new JMenuItem("バージョン情報 (A)..."));
				mnuWebpage.addActionListener(this); mnuWebpage.setMnemonic(KeyEvent.VK_W);
				mnuAbout.addActionListener(this);   mnuAbout.setMnemonic(KeyEvent.VK_A);
		} else {
			setJMenuBar(mnu);
			mnu.add(mnuFile = new JMenu("ファイル"));
				mnuFile.add(mnuLoad = new JMenuItem("設定ファイルを開く..."));
				mnuFile.add(mnuSave = new JMenuItem("設定をファイルに保存..."));
				mnuLoad.addActionListener(this);
				mnuSave.addActionListener(this);
			mnu.add(mnuHelp = new JMenu("ヘルプ"));
				mnuHelp.add(mnuWebpage = new JMenuItem("ブラウザでサポートページを開く"));
				mnuWebpage.addActionListener(this);
		}

		/* ツールバーの設定・表示 */
		java.net.URL dropdown = getClass().getClassLoader().getResource("resources/dropdown.png");
		if (dropdown == null){
			JOptionPane.showMessageDialog(null, "リソース dropdown.png が見つかりません", "エラー", JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
		add(tlb, BorderLayout.NORTH); tlb.setFloatable(false); tlb.add(tlbPanel);
		tlb.setMargin(new Insets(3, 0, 3, 0));
		tlbPanel.setLayout(new BoxLayout(tlbPanel, BoxLayout.X_AXIS));
		tlbPanel.add(Box.createGlue()); tlbPanel.setOpaque(false);
			tlbPanel.add(tlbShuffle); tlbShuffle.setMnemonic(KeyEvent.VK_S); tlbShuffle.addActionListener(this);
				tlbShuffle.setPreferredSize(new Dimension(150, tlbShuffle.getPreferredSize().height));
				tlbShuffle.setMaximumSize(new Dimension(150, Short.MAX_VALUE));
				tlbShuffle.setFocusPainted(false);
			tlbPanel.add(Box.createHorizontalStrut(5));
			tlbPanel.add(tlbLast); tlbLast.setMnemonic(KeyEvent.VK_L); tlbLast.addActionListener(this); tlbLast.setEnabled(false);
				tlbLast.setPreferredSize(new Dimension(138, tlbShuffle.getPreferredSize().height));
				tlbLast.setMaximumSize(new Dimension(138, Short.MAX_VALUE));
				tlbLast.setFocusPainted(false);
			tlbPanel.add(tlbHistory); tlbHistory.addActionListener(this); tlbHistory.setEnabled(false);
				tlbHistory.setIcon(new ImageIcon(dropdown));
				tlbHistory.setPreferredSize(new Dimension(12, tlbShuffle.getPreferredSize().height));
				tlbHistory.setMaximumSize(new Dimension(12, Short.MAX_VALUE));
				tlbHistory.setFocusPainted(false);
			tlbPanel.add(Box.createHorizontalStrut(10));

		/* その他のコンポーネントの設定・表示 */
		java.net.URL logo = getClass().getClassLoader().getResource("resources/logo.png");
		if (logo == null){
			JOptionPane.showMessageDialog(null, "リソース logo.png が見つかりません", "エラー", JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
		add(west, BorderLayout.WEST);
		add(scrollPane, BorderLayout.CENTER); center.setLayout(cardLayout);
		west.setPreferredSize(new Dimension(200, west.getHeight()));
		west.setBackground(new Color(240, 240, 240));
		west.setLayout(new BoxLayout(west, BoxLayout.Y_AXIS));
		west.add(new JLabel(new ImageIcon(logo)));
		for (int i = 0; i < pages.length; i++){
			west.add(pages[i].tab);
			center.add(pages[i], Integer.toString(pages[i].page_no));
		}
		west.add(Box.createVerticalStrut(Integer.MAX_VALUE));
		center.setPreferredSize(new Dimension(0, 0));
		pages[1].setSelected(true);
		switchPage(0);

		dialog = new JFileChooser(DominionSupply.appPath);
		dialog.addChoosableFileFilter(iniFilter);
		dialog.setFileFilter(iniFilter);

		/* 前回保存した設定ファイルの読込 */
		try {
			readSettings(new BufferedReader(new InputStreamReader(new FileInputStream(new File(DominionSupply.appPath, "settings.ini")),"UTF-8")));
		} catch (Exception ex){}

		for (Page page : pages) page.setSelected(page.isSelected());
	}

	/* シャッフル */
	private void shuffle(){
		try {
			Shuffler result = new Shuffler(
				settings.getNeedNumber(),
				settings.getTrash(),
				settings.getAction(),
				settings.getBuy(),
				settings.getBMarket(),
				CardData.ALCHEMY.getAlchemyConstraint()
			);
			if (result.chosen == null) return;
			history.addFirst(result);
			if (history.size() > 10) history.removeLast();
			tlbLast.setEnabled(true);
			tlbHistory.setEnabled(true);
			if (result.alert != null) JOptionPane.showMessageDialog(this, result.alert, "抽出失敗", JOptionPane.WARNING_MESSAGE);
			showResult(result);

			if (settings.getUncheck()){
				for (Card card : result.chosen) card.setSelected(false);
			}
		} catch (ExpectedException ex){
			JOptionPane.showMessageDialog(this, ex.getMessage(), "エラー", JOptionPane.ERROR_MESSAGE);
		} catch (Exception ex){
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this, "予期せぬエラーが発生しました", "エラー", JOptionPane.ERROR_MESSAGE);
		}
	}

	/* ページの切り替え */
	public void switchPage(int page_no){
		Page page;

		if (page_no < 0 || page_no >= CardData.pages.length) return;
		for (int i = 0; i < CardData.pages.length; i++){
			page = CardData.pages[i];
			if (page.page_no == page_no){
				page.setTabColor(true);
				activePage = page;
				cardLayout.show(center, Integer.toString(page_no));
				Dimension preferredSize = page.getPreferredSize();
				center.setPreferredSize(new Dimension(0, preferredSize.height));
			} else {
				page.setTabColor(false);
			}
		}
	}

	/* 結果・履歴の表示 */
	private static Comparator<Card>
		comperator1 = new Comparator<Card>(){ public int compare(Card o1, Card o2){ return o1.order1 - o2.order1; } },
		comperator2 = new Comparator<Card>(){ public int compare(Card o1, Card o2){ return o1.order2 - o2.order2; } };
	private void showResult(Shuffler result){
		Collections.sort(result.chosen, settings.getOrder() ? comperator1 : comperator2);
		(new ShowDialog(this, result, settings.getBMarket())).setVisible(true);
	}
	public void showHistory(){
		JPopupMenu popup = new JPopupMenu();
		int i = 0;
		for (Shuffler result : history){
			Collections.sort(result.chosen, settings.getOrder() ? comperator1 : comperator2);
			String text = String.format("%d. %s", ++i, result.toString());
			popup.add(new HistoryItem(text, result));
		}
		popup.show(tlbLast, 0, tlbLast.getHeight());
	}
	class HistoryItem extends JMenuItem implements ActionListener {
		private Shuffler result;

		public HistoryItem(String text, Shuffler result){
			this.result = result;
			this.setText(text);
			this.addActionListener(this);
		}

		public void actionPerformed(ActionEvent e){
			showResult(result);
		}
	}

	/* 設定をファイルから読込 */
	private void readSettings(BufferedReader file) throws Exception {
		String line;
		Double version = 2.41;
		int activePageNo = 0;

		while ((line = file.readLine()) != null){
			try {
				String eq[] = line.split("=", 2);
				if (eq.length <= 1) continue;
				eq[0] = eq[0].trim().toLowerCase();
				String _right[] = eq[1].split(",");
				int right[] = new int[_right.length];
				for (int i = 0; i < right.length; i++){
					try {
						_right[i] = _right[i].trim();
						right[i] = _right[i].isEmpty() ? 0 : Integer.parseInt(_right[i]);
					} catch (Exception ex){
						right[i] = 1;
					}
				}

				if (eq[0].equals("version")){
					version = Double.parseDouble(eq[1]);
				} else if (eq[0].equals("activepage")){
					activePageNo = right[0];
				} else if (eq[0].equals("number")){
					if (right[0] >= 0 && right[0] <= 10) settings.setNeedNumber(right[0]);
				} else if (eq[0].equals("uncheck")){
					settings.setUncheck(right[0] != 0);
				} else if (eq[0].equals("trash")){
					settings.setTrash(right[0] != 0);
				} else if (eq[0].equals("action")){
					settings.setAction(right[0] != 0);
				} else if (eq[0].equals("buy")){
					settings.setBuy(right[0] != 0);
				} else if (eq[0].equals("order")){
					settings.setOrder(right[0] != 2);
				} else if (eq[0].equals("bmarket")){
					settings.setBMarket(right[0] != 2);
				} else if (eq[0].startsWith("expansion")){
					int page_no = Integer.parseInt(eq[0].substring(9));
					if (version <= 2.0 && page_no == 7) page_no = 8;
					Page page = CardData.pages[page_no];
					page.setSelected(right[0] != 0);
					page.setMin(right[1]);
					page.setMax(right[2]);

					if (page.cards == null) continue;
					int i = 3;
					for (Card card : page.cards) card.setSelected(right[i++] != 0);
				}
			} catch (Exception e){
				continue;
			}
		}
		switchPage(activePageNo);
		file.close();
	}

	/* 設定をファイルに保存 */
	private void writeSettings(PrintWriter file) throws Exception {
		file.printf("version=2.41\r\n");
		file.printf("activepage=%d\r\n", activePage.page_no);
		file.printf("number=%d\r\n", settings.getNeedNumber());
		file.printf("uncheck=%d\r\n", settings.getUncheck() ? 1 : 0);
		file.printf("trash=%s\r\n", settings.getTrash() ? 1 : 0);
		file.printf("action=%s\r\n", settings.getAction() ? 1 : 0);
		file.printf("buy=%s\r\n", settings.getBuy() ? 1 : 0);
		file.printf("order=%d\r\n", settings.getOrder() ? 1 : 2);
		file.printf("bmarket=%d\r\n", settings.getBMarket() ? 1 : 2);
		for (int i = 1; i < CardData.pages.length; i++){
			Page page = CardData.pages[i];
			if (page.cards == null) continue;
			file.printf("expansion%d=%d,%d,%d", i, page.isSelected() ? 1 : 0, page.getMin(), page.getMax());
			for (Card card : page.cards) file.printf(",%d", card.selected ? 1 : 0);
			file.printf("\r\n");
		}
		file.close();
	}

	/* イベントハンドラの設定 */
	public void actionPerformed(ActionEvent e){
		if (e.getSource() == tlbShuffle){
			shuffle();
		} else if (e.getSource() == tlbLast){
			showResult(history.getFirst());
		} else if (e.getSource() == tlbHistory){
			showHistory();
		} else if (e.getSource() == mnuLoad){
			mnuLoad_Click();
		} else if (e.getSource() == mnuSave){
			mnuSave_Click();
		} else if (e.getSource() == mnuExit){
			unload();
		} else if (e.getSource() == mnuWebpage){
			openBrowser();
		} else if (e.getSource() == mnuAbout){
			mnuAbout_Click();
		}
	}
	public void mnuLoad_Click(){
		if (dialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION){
			try {
				readSettings(new BufferedReader(new InputStreamReader(new FileInputStream(dialog.getSelectedFile()),"UTF-8")));
			} catch (Exception e){
				JOptionPane.showMessageDialog(this, "ファイルの読み込みに失敗しました", "エラー", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	public void mnuSave_Click(){
		if (dialog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION){
			try {
				String path = dialog.getSelectedFile().getPath();
				if (dialog.getSelectedFile().getName().indexOf('.') < 0 && dialog.getFileFilter() == iniFilter) path += ".ini";
				writeSettings(new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(path),"UTF-8"))));
			} catch (Exception e){
				JOptionPane.showMessageDialog(this, "ファイルの書き込みに失敗しました", "エラー", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	public void mnuAbout_Click(){
		JOptionPane.showMessageDialog(this, "Dominion サプライ選択装置 2.41\r\n(c) 鵺, 2014", "About", JOptionPane.INFORMATION_MESSAGE);
	}
	public void unload(){
		unloadAdapter.windowClosing(null);
	}

	/* サポートページをブラウザで開く */
	public void openBrowser(){
		final String url = "http://nue2004.info/program/dominionsupply/";

		/* Desktop.getDesktop().browse() が使える場合はそれを使う */
		if (Desktop.isDesktopSupported()){
			if (Desktop.getDesktop().isSupported(java.awt.Desktop.Action.BROWSE)){
				try {
					Desktop.getDesktop().browse(new java.net.URI(url));
					return;
				} catch (Exception ex){
					/* 失敗した場合は OS 毎の操作 */;
				}
			}
		}

		/* そうでない場合は OS 毎の動作（Linux, Windows, Mac OS のみ対応） */
		String os = System.getProperty("os.name");
		try {
			if (os.indexOf("Linux") >= 0){
				Runtime.getRuntime().exec(new String[]{ "/etc/alternatives/x-www-browser", url });
				return;
			} else if (os.indexOf("Windows") >= 0){
				Process process = Runtime.getRuntime().exec(
					"cmd /c for /f \"tokens=3\" %i in ('reg query \"HKEY_CLASSES_ROOT\\http\\shell\\open\\command\" /ve') do @echo %i");
				InputStream is = process.getInputStream();
				String str = (new BufferedReader(new InputStreamReader(is, "Shift_JIS"))).readLine();
				is.close();
				if (str == null) throw new Exception();
				Runtime.getRuntime().exec(new String[]{ str, url });
				return;
			} else if (os.indexOf("Mac") >= 0){
				Runtime.getRuntime().exec(new String[]{ "open", url });
				return;

			}
		} catch (Exception ex){
			ex.printStackTrace();
		}

		JOptionPane.showMessageDialog(this, "ブラウザの起動に失敗しました。", "エラー", JOptionPane.ERROR_MESSAGE);
		return;
	}

	public void componentResized(ComponentEvent e){
		try {	// ad hoc
			Dimension preferredSize = activePage.getPreferredSize();
			center.setPreferredSize(new Dimension(0, preferredSize.height));
			cardLayout.next(center);
			cardLayout.previous(center);
		} catch (Exception ex){}
	}
	public void componentHidden(ComponentEvent e){}
	public void componentMoved(ComponentEvent e){}
	public void componentShown(ComponentEvent e){}

	private WindowAdapter unloadAdapter = new WindowAdapter(){
		public void windowClosing(WindowEvent e){
			try {
				writeSettings(new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(new File(DominionSupply.appPath, "settings.ini")),"UTF-8"))));
			} catch (Exception ex){}
			System.exit(0);
		}
	};
}

