/* 抽出されたカードを表示するダイアログボックス */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class ShowDialog extends JDialog implements ActionListener {
	private JButton btnClose = new JButton("閉じる"), btnBMarket;
	private Shuffler result;
	private boolean bmRange;
	private CardImage baneImg;

	public ShowDialog(Frame owner, Shuffler result, boolean bmRange){
		super(owner, true);
		this.result = result;
		this.bmRange = bmRange;

		setTitle("抽出されたカード");
		setLayout(new BorderLayout());
		getContentPane().setBackground(Color.WHITE);
		setResizable(false);
		getRootPane().setDefaultButton(btnClose);
		setModalityType(Dialog.ModalityType.MODELESS);

		JPanel center = new JPanel(), south = new JPanel();
		int n = result.chosen.size();
		add(center, BorderLayout.CENTER);
		center.setLayout(new GridLayout((n >= 5) ? 2 : 1, (n >= 5) ? (n + 1) / 2 : n, 10, 10));
		center.setOpaque(false);
		center.setBorder(BorderFactory.createEmptyBorder(15, 15, 10, 15));
		for (Card card : result.chosen) center.add((CardImage)card.cardImg.clone());

		add(south, BorderLayout.SOUTH);
		south.setOpaque(false);
		if (result.useColonyAndPlatinum == Shuffler.TriState.NO && result.useShelters == Shuffler.TriState.NO){
			south.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
		} else {
			String str1 =
				result.useColonyAndPlatinum == Shuffler.TriState.TRUE  ? "植民地と白金貨 : 使用する" :
				result.useColonyAndPlatinum == Shuffler.TriState.FALSE ? "植民地と白金貨 : 使用しない" : "",
			str2 =
				result.useShelters == Shuffler.TriState.TRUE  ? "初期手札 : 銅貨と避難所" :
				result.useShelters == Shuffler.TriState.FALSE ? "初期手札 : 銅貨と屋敷" : "";
			JLabel colonyLabel = new JLabel(!(str1.equals("") || str2.equals("")) ? "<html>" + str1 + "<br>" + str2 : str1 + str2, SwingConstants.CENTER);
			south.setLayout(new BorderLayout());
			south.add(colonyLabel, BorderLayout.NORTH);

			JPanel newsouth = new JPanel();
			south.add(newsouth, BorderLayout.CENTER);
			newsouth.setOpaque(false);
			newsouth.setBorder(BorderFactory.createEmptyBorder(5, 0, 10, 0));
			south = newsouth;
		}
		south.setLayout(new FlowLayout(FlowLayout.CENTER));
		south.add(btnClose);
		btnClose.setPreferredSize(new Dimension(125, 25));
		btnClose.addActionListener(this);

		if (result.bmarket){
			btnBMarket = new JButton("闇市場デッキ");
			south.add(btnBMarket);
			btnBMarket.setPreferredSize(new Dimension(125, 25));
			btnBMarket.addActionListener(this);
		}

		if (result.bane != null){
			JPanel east = new JPanel(), banePanel = new JPanel();
			JLabel baneLabel = new JLabel("災いカード", SwingConstants.CENTER);
			GridBagLayout gbl = new GridBagLayout();
			add(east, BorderLayout.EAST);
			east.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 15));
			east.setLayout(gbl);
			east.setOpaque(false);
			gbl.setConstraints(banePanel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
			east.add(banePanel);
			banePanel.setBorder(BorderFactory.createEmptyBorder(5, 10, 10, 10));
			banePanel.setLayout(new BorderLayout());
			banePanel.setBackground(new Color(255, 223, 238));
			banePanel.add(baneLabel, BorderLayout.NORTH);
			banePanel.add(baneImg = (CardImage)result.bane.cardImg.clone(), BorderLayout.CENTER);
			baneLabel.setForeground(Color.RED);
			baneImg.setAlignmentX(Component.CENTER_ALIGNMENT);
			baneImg.setAlignmentY(Component.CENTER_ALIGNMENT);
			baneImg.setSize(baneImg.getPreferredSize());
		}

		pack();
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(screen.x + screen.width / 2  - getSize().width / 2, screen.y + screen.height / 2 - getSize().height / 2);
	}

	public void actionPerformed(ActionEvent e){
		if (e.getSource() == btnClose){
			dispose();
		} else if (e.getSource() == btnBMarket){
			for (Expansion exp : Expansion.allInstances){
				if (!exp.isSelected()) continue;
				for (Card card : exp.cards) card.disabled = false;
			}
			for (Card card : result.chosen) card.disabled = true;
			if (result.bane != null) result.bane.disabled = true;
			(new BrackMarket(bmRange)).setVisible(true);
		}
	}
}
